require_relative "statsman/version"
require "singleton"
require "logger"
require_relative "statsman/reporter_job"
require "json"
require_relative "statsman/accountable"

module Statsman
  
  def config
    yield Config.instance
  end

  def send_counter(key, value, meta={})
    agent = Agent.with_config(Config.instance)
    agent.send_data(:counter, key, value, meta)
  end

  def send_value(key, value, meta={})
    agent = Agent.with_config(Config.instance)
    agent.send_data(:value, key, value, meta)
  end


  module_function :config
  module_function :send_counter
  module_function :send_value

  class Config
    include Singleton
    attr_accessor :api_key, :api_secret, :url, :logging_on
  end # Config

  class Agent
    include Singleton
    attr_accessor :config

    def self.logger
      if defined? Rails
        log_path = "log/statsman.#{Rails.env}.log"
      else
        log_path = "./statsman.log"
      end

      @logger ||= Logger.new(log_path)
    end

    def self.with_config(config)
      i = instance
      i.config = config
      i
    end

    def self.log(str)
      logger.info(str) if Config.instance.logging_on
    end

    def log(str)
      self.class.log(str)
    end

    def send_data(data_type, key, value, meta = {})
      log("enqueueing data_type: #{data_type}, key: #{key}, value: #{value}, meta: #{meta.inspect}")
      Statsman::ReporterJob.new.async.perform(config, data_type, key, value, meta.to_json)
    end

  end # Agent

end

